package com.hw.wms.controller;

import java.util.List;
import java.io.IOException;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.hw.common.log.annotation.Log;
import com.hw.common.log.enums.BusinessType;
import com.hw.common.security.annotation.RequiresPermissions;
import com.hw.wms.domain.WmsProductInstock;
import com.hw.wms.service.IWmsProductInstockService;
import com.hw.common.core.web.controller.BaseController;
import com.hw.common.core.web.domain.AjaxResult;
import com.hw.common.core.utils.poi.ExcelUtil;
import com.hw.common.core.web.page.TableDataInfo;

/**
 * 成品入库记录;成品入库记录Controller
 * 
 * @author xins
 * @date 2023-12-25
 */
@RestController
@RequestMapping("/productinstock")
public class WmsProductInstockController extends BaseController
{
    @Autowired
    private IWmsProductInstockService wmsProductInstockService;

    /**
     * 查询成品入库记录;成品入库记录列表
     */
    @RequiresPermissions("wms:productinstock:list")
    @GetMapping("/list")
    public TableDataInfo list(WmsProductInstock wmsProductInstock)
    {
        startPage();
        List<WmsProductInstock> list = wmsProductInstockService.selectWmsProductInstockList(wmsProductInstock);
        return getDataTable(list);
    }

    /**
     * 导出成品入库记录;成品入库记录列表
     */
    @RequiresPermissions("wms:productinstock:export")
    @Log(title = "成品入库记录;成品入库记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, WmsProductInstock wmsProductInstock)
    {
        List<WmsProductInstock> list = wmsProductInstockService.selectWmsProductInstockList(wmsProductInstock);
        ExcelUtil<WmsProductInstock> util = new ExcelUtil<WmsProductInstock>(WmsProductInstock.class);
        util.exportExcel(response, list, "成品入库记录;成品入库记录数据");
    }

    /**
     * 获取成品入库记录;成品入库记录详细信息
     */
    @RequiresPermissions("wms:productinstock:query")
    @GetMapping(value = "/{productInstockId}")
    public AjaxResult getInfo(@PathVariable("productInstockId") Long productInstockId)
    {
        return success(wmsProductInstockService.selectWmsProductInstockByProductInstockId(productInstockId));
    }

    /**
     * 新增成品入库记录;成品入库记录
     */
    @RequiresPermissions("wms:productinstock:add")
    @Log(title = "成品入库记录;成品入库记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody WmsProductInstock wmsProductInstock)
    {
        return toAjax(wmsProductInstockService.insertWmsProductInstock(wmsProductInstock));
    }

    /**
     * 修改成品入库记录;成品入库记录
     */
    @RequiresPermissions("wms:productinstock:edit")
    @Log(title = "成品入库记录;成品入库记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody WmsProductInstock wmsProductInstock)
    {
        return toAjax(wmsProductInstockService.updateWmsProductInstock(wmsProductInstock));
    }

    /**
     * 删除成品入库记录;成品入库记录
     */
    @RequiresPermissions("wms:productinstock:remove")
    @Log(title = "成品入库记录;成品入库记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{productInstockIds}")
    public AjaxResult remove(@PathVariable Long[] productInstockIds)
    {
        return toAjax(wmsProductInstockService.deleteWmsProductInstockByProductInstockIds(productInstockIds));
    }
}
